@@ -1,7 +1,7 @@
 /*
  * jab_iq.c     -- Jabber protocol IQ-related fonctions
  *
- * Copyright (C) 2005-2007 Mikael Berthe <mikael@lilotux.net>
+ * Copyright (C) 2005-2008 Mikael Berthe <mikael@lilotux.net>
  * Some parts initially came from the centericq project:
  * Copyright (C) 2002-2005 by Konstantin Klyagin <konst@konst.org.ua>
  * Some small parts come from the Pidgin project <http://pidgin.im/>
@@ -1069,12 +1069,18 @@ static void handle_iq_commands_list(jconn conn, char *from, const char *id,
   xmlnode myquery;
   jid requester_jid;
   const struct adhoc_command *command;
+  const char *node;
   bool from_self;
+
   x = jutil_iqnew(JPACKET__RESULT, NS_DISCO_ITEMS);
   xmlnode_put_attrib(x, "id", id);
   xmlnode_put_attrib(x, "to", xmlnode_get_attrib(xmldata, "from"));
   myquery = xmlnode_get_tag(x, "query");
 
+  node = xmlnode_get_attrib(xmlnode_get_tag(xmldata, "query"), "node");
+  if (node)
+    xmlnode_put_attrib(myquery, "node", node);
+
   requester_jid = jid_new(conn->p, xmlnode_get_attrib(xmldata, "from"));
   from_self = !jid_cmpx(conn->user, requester_jid, JID_USER | JID_SERVER);
 
@@ -1094,15 +1100,17 @@ static void handle_iq_commands_list(jconn conn, char *from, const char *id,
 
 static void xmlnode_insert_dataform_result_message(xmlnode node, char *message)
 {
-  xmlnode x = xmlnode_insert_tag(node, "x");
+  xmlnode x, field, value;
+
+  x = xmlnode_insert_tag(node, "x");
   xmlnode_put_attrib(x, "type", "result");
   xmlnode_put_attrib(x, "xmlns", "jabber:x:data");
 
-  xmlnode field = xmlnode_insert_tag(x, "field");
+  field = xmlnode_insert_tag(x, "field");
   xmlnode_put_attrib(field, "type", "text-single");
   xmlnode_put_attrib(field, "var", "message");
 
-  xmlnode value = xmlnode_insert_tag(field, "value");
+  value = xmlnode_insert_tag(field, "value");
   xmlnode_insert_cdata(value, message, -1);
 }
 
@@ -1111,7 +1119,7 @@ static char *generate_session_id(char *prefix)
   char *result;
   static int counter = 0;
   counter++;
-  // TODO better use timezone ?
+  // TODO better use timestamp?
   result = g_strdup_printf("%s-%i", prefix, counter);
   return result;
 }
@@ -1134,9 +1142,12 @@ static void handle_iq_command_set_status(jconn conn, char *from, const char *id,
   xmlnode_put_attrib(command, "xmlns", NS_COMMANDS);
 
   if (!sessionid) {
+    xmlnode value;
+
     sessionid = generate_session_id("set-status");
     xmlnode_put_attrib(command, "sessionid", sessionid);
     g_free(sessionid);
+    sessionid = NULL;
     xmlnode_put_attrib(command, "status", "executing");
 
     x = xmlnode_insert_tag(command, "x");
@@ -1155,7 +1166,7 @@ static void handle_iq_command_set_status(jconn conn, char *from, const char *id,
     xmlnode_put_attrib(y, "type", "hidden");
     xmlnode_put_attrib(y, "var", "FORM_TYPE");
 
-    xmlnode value = xmlnode_insert_tag(y, "value");
+    value = xmlnode_insert_tag(y, "value");
     xmlnode_insert_cdata(value, "http://jabber.org/protocol/rc", -1);
 
     y = xmlnode_insert_tag(x, "field");
@@ -1180,9 +1191,9 @@ static void handle_iq_command_set_status(jconn conn, char *from, const char *id,
     xmlnode_put_attrib(y, "type", "text-multi");
     xmlnode_put_attrib(y, "var", "status-message");
     xmlnode_put_attrib(y, "label", "Message");
-  }
-  else // (if sessionid)
-  {
+  } else if (!strcmp(action, "cancel")) {
+    xmlnode_put_attrib(command, "status", "canceled");
+  } else  { // (if sessionid and not canceled)
     y = xmlnode_get_tag(x, "x?xmlns=jabber:x:data");
     if (y) {
       char *value, *message;
@@ -1194,7 +1205,7 @@ static void handle_iq_command_set_status(jconn conn, char *from, const char *id,
       if (s->name) {
         char *status = g_strdup_printf("%s %s", s->status,
                                        message ? message : "");
-        setstatus(NULL, status);
+        cmd_setstatus(NULL, status);
         g_free(status);
         xmlnode_put_attrib(command, "status", "completed");
         xmlnode_put_attrib(iq, "type", "result");
@@ -1203,6 +1214,8 @@ static void handle_iq_command_set_status(jconn conn, char *from, const char *id,
       }
     }
   }
+  if (sessionid)
+    xmlnode_put_attrib(command, "sessionid", sessionid);
   xmlnode_put_attrib(iq, "to", xmlnode_get_attrib(xmldata, "from"));
   xmlnode_put_attrib(iq, "id", id);
   jab_send(jc, iq);
@@ -1211,7 +1224,7 @@ static void handle_iq_command_set_status(jconn conn, char *from, const char *id,
 
 static void _callback_foreach_buddy_groupchat(gpointer rosterdata, void *param)
 {
-  xmlnode value;
+  xmlnode value, option;
   xmlnode *field;
   const char *room_jid, *nickname;
   char *desc;
@@ -1222,7 +1235,7 @@ static void _callback_foreach_buddy_groupchat(gpointer rosterdata, void *param)
   if (!nickname) return;
   field = param;
 
-  xmlnode option = xmlnode_insert_tag(*field, "option");
+  option = xmlnode_insert_tag(*field, "option");
   value = xmlnode_insert_tag(option, "value");
   xmlnode_insert_cdata(value, room_jid, -1);
   desc = g_strdup_printf("%s on %s", nickname, room_jid);
@@ -1247,27 +1260,30 @@ static void handle_iq_command_leave_groupchats(jconn conn, char *from,
   xmlnode_put_attrib(command, "xmlns", NS_COMMANDS);
 
   if (!sessionid) {
+    xmlnode title, instructions, field, value;
+
     sessionid = generate_session_id("leave-groupchats");
     xmlnode_put_attrib(command, "sessionid", sessionid);
     g_free(sessionid);
+    sessionid = NULL;
     xmlnode_put_attrib(command, "status", "executing");
 
     x = xmlnode_insert_tag(command, "x");
     xmlnode_put_attrib(x, "type", "form");
     xmlnode_put_attrib(x, "xmlns", "jabber:x:data");
 
-    xmlnode title = xmlnode_insert_tag(x, "title");
+    title = xmlnode_insert_tag(x, "title");
     xmlnode_insert_cdata(title, "Leave groupchat(s)", -1);
 
-    xmlnode instructions = xmlnode_insert_tag(x, "instructions");
+    instructions = xmlnode_insert_tag(x, "instructions");
     xmlnode_insert_cdata(instructions, "What groupchats do you want to leave?",
                          -1);
 
-    xmlnode field = xmlnode_insert_tag(x, "field");
+    field = xmlnode_insert_tag(x, "field");
     xmlnode_put_attrib(field, "type", "hidden");
     xmlnode_put_attrib(field, "var", "FORM_TYPE");
 
-    xmlnode value = xmlnode_insert_tag(field, "value");
+    value = xmlnode_insert_tag(field, "value");
     xmlnode_insert_cdata(value, "http://jabber.org/protocol/rc", -1);
 
     field = xmlnode_insert_tag(x, "field");
@@ -1277,14 +1293,15 @@ static void handle_iq_command_leave_groupchats(jconn conn, char *from,
     xmlnode_insert_tag(field, "required");
 
     foreach_buddy(ROSTER_TYPE_ROOM, &_callback_foreach_buddy_groupchat, &field);
-  }
-  else // (if sessionid)
-  {
+  } else if (!strcmp(action, "cancel")) {
+    xmlnode_put_attrib(command, "status", "canceled");
+  } else  { // (if sessionid and not canceled)
     xmlnode form = xmlnode_get_tag(x, "x?xmlns=jabber:x:data");
     if (form) {
+      xmlnode x, gc;
+
       xmlnode_put_attrib(command, "status", "completed");
-      xmlnode gc = xmlnode_get_tag(form, "field?var=groupchats");
-      xmlnode x;
+      gc = xmlnode_get_tag(form, "field?var=groupchats");
 
       for (x = xmlnode_get_firstchild(gc) ; x ; x = xmlnode_get_nextsibling(x))
       {
@@ -1292,7 +1309,7 @@ static void handle_iq_command_leave_groupchats(jconn conn, char *from,
         if (to_leave) {
           GList* b = buddy_search_jid(to_leave);
           if (b)
-            room_leave(b->data, "Asked by remote command");
+            cmd_room_leave(b->data, "Requested by remote command");
         }
       }
       xmlnode_put_attrib(iq, "type", "result");
@@ -1300,6 +1317,8 @@ static void handle_iq_command_leave_groupchats(jconn conn, char *from,
                                              "Groupchats have been left");
     }
   }
+  if (sessionid)
+    xmlnode_put_attrib(command, "sessionid", sessionid);
   xmlnode_put_attrib(iq, "to", xmlnode_get_attrib(xmldata, "from"));
   xmlnode_put_attrib(iq, "id", id);
   jab_send(jc, iq);
@@ -1341,7 +1360,7 @@ static void handle_iq_disco_items(jconn conn, char *from, const char *id,
                                   xmlnode xmldata)
 {
   xmlnode x;
-  char *node;
+  const char *node;
   x = xmlnode_get_tag(xmldata, "query");
   node = xmlnode_get_attrib(x, "node");
   if (node) {
@@ -1461,8 +1480,10 @@ static void handle_iq_last(jconn conn, char *from, const char *id,
   xmlnode myquery;
   char *seconds;
 
-  scr_LogPrint(LPRINT_LOGNORM, "Received an IQ last time request from <%s>",
-               from);
+  if (!settings_opt_get_int("iq_hide_requests")) {
+    scr_LogPrint(LPRINT_LOGNORM, "Received an IQ last time request from <%s>",
+                 from);
+  }
 
   x = jutil_iqnew(JPACKET__RESULT, NS_LAST);
   xmlnode_put_attrib(x, "id", id);
@@ -1492,8 +1513,10 @@ static void handle_iq_version(jconn conn, char *from, const char *id,
   char *os = NULL;
   char *ver = mcabber_version();
 
-  scr_LogPrint(LPRINT_LOGNORM, "Received an IQ version request from <%s>",
-               from);
+  if (!settings_opt_get_int("iq_hide_requests")) {
+    scr_LogPrint(LPRINT_LOGNORM, "Received an IQ version request from <%s>",
+                 from);
+  }
 
   if (!settings_opt_get_int("iq_version_hide_os")) {
     struct utsname osinfo;
@@ -1531,7 +1554,9 @@ static void handle_iq_time(jconn conn, char *from, const char *id,
 
   time(&now_t);
 
-  scr_LogPrint(LPRINT_LOGNORM, "Received an IQ time request from <%s>", from);
+  if (!settings_opt_get_int("iq_hide_requests")) {
+    scr_LogPrint(LPRINT_LOGNORM, "Received an IQ time request from <%s>", from);
+  }
 
   buf = g_new0(char, 512);
 
@@ -1578,7 +1603,9 @@ static void handle_iq_time202(jconn conn, char *from, const char *id,
 
   time(&now_t);
 
-  scr_LogPrint(LPRINT_LOGNORM, "Received an IQ time request from <%s>", from);
+  if (!settings_opt_get_int("iq_hide_requests")) {
+    scr_LogPrint(LPRINT_LOGNORM, "Received an IQ time request from <%s>", from);
+  }
 
   buf = g_new0(char, 512);
 
@@ -1696,6 +1723,7 @@ static void handle_iq_set(jconn conn, char *from, xmlnode xmldata)
     ns = xmlnode_get_attrib(x, "xmlns");
     if (ns && !strcmp(ns, NS_COMMANDS)) {
       handle_iq_commands(conn, from, id, xmldata);
+      return;
     } else {
       iq_not_implemented = TRUE;
     }
