@@ -1,7 +1,7 @@
 /*
  * roster.c     -- Local roster implementation
  *
- * Copyright (C) 2005-2007 Mikael Berthe <mikael@lilotux.net>
+ * Copyright (C) 2005-2008 Mikael Berthe <mikael@lilotux.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,9 @@
 
 #include "roster.h"
 #include "utils.h"
+#include "hooks.h"
 
+extern void hlog_save_state(void);
 
 char *strrole[] = {   /* Should match enum in roster.h */
   "none",
@@ -118,8 +120,7 @@ GList *alternate_buddy;
 
 static roster roster_special;
 
-void unread_jid_add(const char *jid);
-int  unread_jid_del(const char *jid);
+static int  unread_jid_del(const char *jid);
 
 #define DFILTER_ALL     63
 #define DFILTER_ONLINE  62
@@ -434,7 +435,8 @@ void roster_del_user(const char *jid)
   roster *roster_usr;
   GSList *node;
 
-  sl_user = roster_find(jid, jidsearch, ROSTER_TYPE_USER|ROSTER_TYPE_AGENT);
+  sl_user = roster_find(jid, jidsearch,
+                        ROSTER_TYPE_USER|ROSTER_TYPE_AGENT|ROSTER_TYPE_ROOM);
   if (sl_user == NULL)
     return;
   roster_usr = (roster*)sl_user->data;
@@ -589,6 +591,7 @@ void roster_msg_setflag(const char *jid, guint special, guint value)
   GSList *sl_user;
   roster *roster_usr, *roster_grp;
   int new_roster_item = FALSE;
+  guint unread_list_modified = FALSE;
 
   if (special) {
     //sl_user = roster_find(jid, namesearch, ROSTER_TYPE_SPECIAL);
@@ -621,6 +624,8 @@ void roster_msg_setflag(const char *jid, guint special, guint value)
   roster_usr = (roster*)sl_user->data;
   roster_grp = (roster*)roster_usr->list->data;
   if (value) {
+    if (!(roster_usr->flags & ROSTER_FLAG_MSG))
+      unread_list_modified = TRUE;
     // Message flag is TRUE.  This is easy, we just have to set both flags
     // to TRUE...
     roster_usr->flags |= ROSTER_FLAG_MSG;
@@ -631,6 +636,8 @@ void roster_msg_setflag(const char *jid, guint special, guint value)
   } else {
     // Message flag is FALSE.
     guint msg = FALSE;
+    if (roster_usr->flags & ROSTER_FLAG_MSG)
+      unread_list_modified = TRUE;
     roster_usr->flags &= ~ROSTER_FLAG_MSG;
     if (unread_list) {
       GSList *node = g_slist_find(unread_list, roster_usr);
@@ -659,6 +666,13 @@ void roster_msg_setflag(const char *jid, guint special, guint value)
 
   if (buddylist && (new_roster_item || !g_list_find(buddylist, roster_usr)))
     buddylist_build();
+
+  if (unread_list_modified) {
+    guint unread_count = g_slist_length(unread_list);
+    hlog_save_state();
+    /* Call external command */
+    hk_ext_cmd("", 'U', (guchar)MIN(255, unread_count), NULL);
+  }
 }
 
 const char *roster_getname(const char *jid)
@@ -794,11 +808,16 @@ void buddylist_set_hide_offline_buddies(int hide)
   }
 }
 
-inline int buddylist_isset_filter(void)
+int buddylist_isset_filter(void)
 {
   return (display_filter != DFILTER_ALL);
 }
 
+int buddylist_is_status_filtered(enum imstatus status)
+{
+  return display_filter & (1 << status);
+}
+
 void buddylist_set_filter(guchar filter)
 {
   display_filter = filter;
@@ -855,7 +874,8 @@ void buddylist_build(void)
       // - group isn't hidden (shrunk)
       // - this is the current_buddy
       if (roster_usrelt == roster_current_buddy ||
-          display_filter & 1<<buddy_getstatus((gpointer)roster_usrelt, NULL) ||
+          buddylist_is_status_filtered(buddy_getstatus((gpointer)roster_usrelt,
+                                                       NULL)) ||
           (buddy_getflags((gpointer)roster_usrelt) &
                (ROSTER_FLAG_LOCK | ROSTER_FLAG_USRLOCK | ROSTER_FLAG_MSG))) {
         // This user should be added.  Maybe the group hasn't been added yet?
@@ -902,9 +922,9 @@ void buddy_hide_group(gpointer rosterdata, int hide)
 
 const char *buddy_getjid(gpointer rosterdata)
 {
+  roster *roster_usr = rosterdata;
   if (!rosterdata)
     return NULL;
-  roster *roster_usr = rosterdata;
   return roster_usr->jid;
 }
 
@@ -1540,11 +1560,36 @@ void unread_jid_add(const char *jid)
 
 //  unread_jid_del(jid)
 // Return TRUE if jid is found in the table (and remove it), FALSE if not
-int unread_jid_del(const char *jid)
+static int unread_jid_del(const char *jid)
 {
   if (!unread_jids)
     return FALSE;
   return g_hash_table_remove(unread_jids, jid);
 }
 
+// Helper function for unread_jid_get_list()
+static void add_to_unreadjids(gpointer key, gpointer value, gpointer udata)
+{
+  GList **listp = udata;
+  *listp = g_list_append(*listp, key);
+}
+
+//  unread_jid_get_list()
+// Return the JID list.
+// The content of the list should not be modified or freed.
+// The caller should call g_list_free() after use.
+GList *unread_jid_get_list(void)
+{
+  GList *list = NULL;
+
+  if (!unread_jids)
+    return NULL;
+
+  // g_hash_table_get_keys() is only in glib >= 2.14
+  //return g_hash_table_get_keys(unread_jids);
+
+  g_hash_table_foreach(unread_jids, add_to_unreadjids, &list);
+  return list;
+}
+
 /* vim: set expandtab cindent cinoptions=>2\:2(0:  For Vim users... */
